/* ------------------------------------------------------------
author: "Grame"
copyright: "(c)GRAME 2006"
license: "BSD"
name: "matrix"
version: "1.0"
Code generated with Faust 2.74.6 (https://faust.grame.fr)
Compilation options: -a puredata.cpp -lang cpp -ct 1 -es 1 -mcd 16 -mdd 1024 -mdy 33 -single -ftz 0
------------------------------------------------------------ */

#ifndef  __mydsp_H__
#define  __mydsp_H__

/************************************************************************
 ************************************************************************
    FAUST Architecture File
    Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
    ---------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation; either version 2.1 of the 
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA. 
 ************************************************************************
 ************************************************************************/

/* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
   This was derived from minimal.cpp included in the Faust distribution.
   Please note that this is to be compiled as a shared library, which is
   then loaded dynamically by Pd as an external. */

#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "faust/misc.h"
#include "faust/gui/UI.h"
#include "faust/gui/meta.h"
#include "faust/dsp/dsp.h"

/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/***************************************************************************
   Pd UI interface
 ***************************************************************************/

enum ui_elem_type_t {
  UI_BUTTON, UI_CHECK_BUTTON,
  UI_V_SLIDER, UI_H_SLIDER, UI_NUM_ENTRY,
  UI_V_BARGRAPH, UI_H_BARGRAPH,
  UI_END_GROUP, UI_V_GROUP, UI_H_GROUP, UI_T_GROUP
};

struct ui_elem_t {
  ui_elem_type_t type;
  char *label;
  float *zone;
  float init, min, max, step;
};

class PdUI : public UI
{
public:
  const char *name;
  int nelems, level;
  ui_elem_t *elems;
		
  PdUI();
  PdUI(const char *nm, const char *s);
  virtual ~PdUI();

protected:
  std::string path;
  void add_elem(ui_elem_type_t type, const char *label = NULL);
  void add_elem(ui_elem_type_t type, const char *label, float *zone);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float init, float min, float max, float step);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float min, float max);

public:
  virtual void addButton(const char* label, float* zone);
  virtual void addCheckButton(const char* label, float* zone);
  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);

  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
    
  virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) {}
  
  virtual void openTabBox(const char* label);
  virtual void openHorizontalBox(const char* label);
  virtual void openVerticalBox(const char* label);
  virtual void closeBox();
	
  virtual void run();
};

static std::string mangle(const char *name, int level, const char *s)
{
  const char *s0 = s;
  std::string t = "";
  if (!s) return t;
  // Get rid of bogus "0x00" labels in recent Faust revisions. Also, for
  // backward compatibility with old Faust versions, make sure that default
  // toplevel groups and explicit toplevel groups with an empty label are
  // treated alike (these both return "0x00" labels in the latest Faust, but
  // would be treated inconsistently in earlier versions).
  if (!*s || strcmp(s, "0x00") == 0) {
    if (level == 0)
      // toplevel group with empty label, map to dsp name
      s = name;
    else
      // empty label
      s = "";
  }
  while (*s)
    if (isalnum(*s))
      t += *(s++);
    else {
      const char *s1 = s;
      while (*s && !isalnum(*s)) ++s;
      if (s1 != s0 && *s) t += "-";
    }
  return t;
}

static std::string normpath(std::string path)
{
  path = std::string("/")+path;
  int pos = path.find("//");
  while (pos >= 0) {
    path.erase(pos, 1);
    pos = path.find("//");
  }
  size_t len = path.length();
  if (len > 1 && path[len-1] == '/')
    path.erase(len-1, 1);
  return path;
}

static std::string pathcat(std::string path, std::string label)
{
  if (path.empty())
    return normpath(label);
  else if (label.empty())
    return normpath(path);
  else
    return normpath(path+"/"+label);
}

PdUI::PdUI()
{
  nelems = level = 0;
  elems = NULL;
  name = "";
  path = "";
}

PdUI::PdUI(const char *nm, const char *s)
{
  nelems = level = 0;
  elems = NULL;
  name = nm?nm:"";
  path = s?s:"";
}

PdUI::~PdUI()
{
  if (elems) {
    for (int i = 0; i < nelems; i++)
      if (elems[i].label)
	free(elems[i].label);
    free(elems);
  }
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = NULL;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 0.0;
  elems[nelems].step = 0.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 1.0;
  elems[nelems].step = 1.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float init, float min, float max, float step)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = init;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = step;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float min, float max)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = 0.0;
  nelems++;
}

void PdUI::addButton(const char* label, float* zone)
{ add_elem(UI_BUTTON, label, zone); }
void PdUI::addCheckButton(const char* label, float* zone)
{ add_elem(UI_CHECK_BUTTON, label, zone); }
void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }

void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_H_BARGRAPH, label, zone, min, max); }
void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_V_BARGRAPH, label, zone, min, max); }

void PdUI::openTabBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openHorizontalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openVerticalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::closeBox()
{
  int pos = path.rfind("/");
  if (pos < 0) pos = 0;
  path.erase(pos);
  level--;
}

void PdUI::run() {}

/******************************************************************************
*******************************************************************************

			    FAUST DSP

*******************************************************************************
*******************************************************************************/

//----------------------------------------------------------------------------
//  FAUST generated signal processor
//----------------------------------------------------------------------------
		
#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif 

#include <algorithm>
#include <cmath>
#include <cstdint>

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

#ifdef __APPLE__ 
#define exp10f __exp10f
#define exp10 __exp10
#endif

#if defined(_WIN32)
#define RESTRICT __restrict
#else
#define RESTRICT __restrict__
#endif


class mydsp : public dsp {
	
 private:
	
	FAUSTFLOAT fVslider0;
	FAUSTFLOAT fVslider1;
	FAUSTFLOAT fVslider2;
	FAUSTFLOAT fVslider3;
	FAUSTFLOAT fVslider4;
	FAUSTFLOAT fVslider5;
	FAUSTFLOAT fVslider6;
	FAUSTFLOAT fVslider7;
	FAUSTFLOAT fVslider8;
	FAUSTFLOAT fVslider9;
	FAUSTFLOAT fVslider10;
	FAUSTFLOAT fVslider11;
	FAUSTFLOAT fVslider12;
	FAUSTFLOAT fVslider13;
	FAUSTFLOAT fVslider14;
	FAUSTFLOAT fVslider15;
	FAUSTFLOAT fVslider16;
	FAUSTFLOAT fVslider17;
	FAUSTFLOAT fVslider18;
	FAUSTFLOAT fVslider19;
	FAUSTFLOAT fVslider20;
	FAUSTFLOAT fVslider21;
	FAUSTFLOAT fVslider22;
	FAUSTFLOAT fVslider23;
	FAUSTFLOAT fVslider24;
	FAUSTFLOAT fVslider25;
	FAUSTFLOAT fVslider26;
	FAUSTFLOAT fVslider27;
	FAUSTFLOAT fVslider28;
	FAUSTFLOAT fVslider29;
	FAUSTFLOAT fVslider30;
	FAUSTFLOAT fVslider31;
	FAUSTFLOAT fVslider32;
	FAUSTFLOAT fVslider33;
	FAUSTFLOAT fVslider34;
	FAUSTFLOAT fVslider35;
	FAUSTFLOAT fVslider36;
	FAUSTFLOAT fVslider37;
	FAUSTFLOAT fVslider38;
	FAUSTFLOAT fVslider39;
	FAUSTFLOAT fVslider40;
	FAUSTFLOAT fVslider41;
	FAUSTFLOAT fVslider42;
	FAUSTFLOAT fVslider43;
	FAUSTFLOAT fVslider44;
	FAUSTFLOAT fVslider45;
	FAUSTFLOAT fVslider46;
	FAUSTFLOAT fVslider47;
	FAUSTFLOAT fVslider48;
	FAUSTFLOAT fVslider49;
	FAUSTFLOAT fVslider50;
	FAUSTFLOAT fVslider51;
	FAUSTFLOAT fVslider52;
	FAUSTFLOAT fVslider53;
	FAUSTFLOAT fVslider54;
	FAUSTFLOAT fVslider55;
	FAUSTFLOAT fVslider56;
	FAUSTFLOAT fVslider57;
	FAUSTFLOAT fVslider58;
	FAUSTFLOAT fVslider59;
	FAUSTFLOAT fVslider60;
	FAUSTFLOAT fVslider61;
	FAUSTFLOAT fVslider62;
	FAUSTFLOAT fVslider63;
	int fSampleRate;
	
 public:
	mydsp() {
	}
	
	void metadata(Meta* m) { 
		m->declare("author", "Grame");
		m->declare("compile_options", "-a puredata.cpp -lang cpp -ct 1 -es 1 -mcd 16 -mdd 1024 -mdy 33 -single -ftz 0");
		m->declare("copyright", "(c)GRAME 2006");
		m->declare("filename", "matrix.dsp");
		m->declare("license", "BSD");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/version", "1.0");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/version", "1.0");
		m->declare("name", "matrix");
		m->declare("version", "1.0");
	}

	virtual int getNumInputs() {
		return 8;
	}
	virtual int getNumOutputs() {
		return 8;
	}
	
	static void classInit(int sample_rate) {
	}
	
	virtual void instanceConstants(int sample_rate) {
		fSampleRate = sample_rate;
	}
	
	virtual void instanceResetUserInterface() {
		fVslider0 = FAUSTFLOAT(-1e+01f);
		fVslider1 = FAUSTFLOAT(-1e+01f);
		fVslider2 = FAUSTFLOAT(-1e+01f);
		fVslider3 = FAUSTFLOAT(-1e+01f);
		fVslider4 = FAUSTFLOAT(-1e+01f);
		fVslider5 = FAUSTFLOAT(-1e+01f);
		fVslider6 = FAUSTFLOAT(-1e+01f);
		fVslider7 = FAUSTFLOAT(-1e+01f);
		fVslider8 = FAUSTFLOAT(-1e+01f);
		fVslider9 = FAUSTFLOAT(-1e+01f);
		fVslider10 = FAUSTFLOAT(-1e+01f);
		fVslider11 = FAUSTFLOAT(-1e+01f);
		fVslider12 = FAUSTFLOAT(-1e+01f);
		fVslider13 = FAUSTFLOAT(-1e+01f);
		fVslider14 = FAUSTFLOAT(-1e+01f);
		fVslider15 = FAUSTFLOAT(-1e+01f);
		fVslider16 = FAUSTFLOAT(-1e+01f);
		fVslider17 = FAUSTFLOAT(-1e+01f);
		fVslider18 = FAUSTFLOAT(-1e+01f);
		fVslider19 = FAUSTFLOAT(-1e+01f);
		fVslider20 = FAUSTFLOAT(-1e+01f);
		fVslider21 = FAUSTFLOAT(-1e+01f);
		fVslider22 = FAUSTFLOAT(-1e+01f);
		fVslider23 = FAUSTFLOAT(-1e+01f);
		fVslider24 = FAUSTFLOAT(-1e+01f);
		fVslider25 = FAUSTFLOAT(-1e+01f);
		fVslider26 = FAUSTFLOAT(-1e+01f);
		fVslider27 = FAUSTFLOAT(-1e+01f);
		fVslider28 = FAUSTFLOAT(-1e+01f);
		fVslider29 = FAUSTFLOAT(-1e+01f);
		fVslider30 = FAUSTFLOAT(-1e+01f);
		fVslider31 = FAUSTFLOAT(-1e+01f);
		fVslider32 = FAUSTFLOAT(-1e+01f);
		fVslider33 = FAUSTFLOAT(-1e+01f);
		fVslider34 = FAUSTFLOAT(-1e+01f);
		fVslider35 = FAUSTFLOAT(-1e+01f);
		fVslider36 = FAUSTFLOAT(-1e+01f);
		fVslider37 = FAUSTFLOAT(-1e+01f);
		fVslider38 = FAUSTFLOAT(-1e+01f);
		fVslider39 = FAUSTFLOAT(-1e+01f);
		fVslider40 = FAUSTFLOAT(-1e+01f);
		fVslider41 = FAUSTFLOAT(-1e+01f);
		fVslider42 = FAUSTFLOAT(-1e+01f);
		fVslider43 = FAUSTFLOAT(-1e+01f);
		fVslider44 = FAUSTFLOAT(-1e+01f);
		fVslider45 = FAUSTFLOAT(-1e+01f);
		fVslider46 = FAUSTFLOAT(-1e+01f);
		fVslider47 = FAUSTFLOAT(-1e+01f);
		fVslider48 = FAUSTFLOAT(-1e+01f);
		fVslider49 = FAUSTFLOAT(-1e+01f);
		fVslider50 = FAUSTFLOAT(-1e+01f);
		fVslider51 = FAUSTFLOAT(-1e+01f);
		fVslider52 = FAUSTFLOAT(-1e+01f);
		fVslider53 = FAUSTFLOAT(-1e+01f);
		fVslider54 = FAUSTFLOAT(-1e+01f);
		fVslider55 = FAUSTFLOAT(-1e+01f);
		fVslider56 = FAUSTFLOAT(-1e+01f);
		fVslider57 = FAUSTFLOAT(-1e+01f);
		fVslider58 = FAUSTFLOAT(-1e+01f);
		fVslider59 = FAUSTFLOAT(-1e+01f);
		fVslider60 = FAUSTFLOAT(-1e+01f);
		fVslider61 = FAUSTFLOAT(-1e+01f);
		fVslider62 = FAUSTFLOAT(-1e+01f);
		fVslider63 = FAUSTFLOAT(-1e+01f);
	}
	
	virtual void instanceClear() {
	}
	
	virtual void init(int sample_rate) {
		classInit(sample_rate);
		instanceInit(sample_rate);
	}
	
	virtual void instanceInit(int sample_rate) {
		instanceConstants(sample_rate);
		instanceResetUserInterface();
		instanceClear();
	}
	
	virtual mydsp* clone() {
		return new mydsp();
	}
	
	virtual int getSampleRate() {
		return fSampleRate;
	}
	
	virtual void buildUserInterface(UI* ui_interface) {
		ui_interface->openTabBox("Matrix 8 x 8");
		ui_interface->openHorizontalBox("Output 0");
		ui_interface->addVerticalSlider("Input 0", &fVslider7, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider6, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider5, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider4, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider3, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider2, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider1, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider0, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 1");
		ui_interface->addVerticalSlider("Input 0", &fVslider15, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider14, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider13, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider12, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider11, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider10, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider9, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider8, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 2");
		ui_interface->addVerticalSlider("Input 0", &fVslider23, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider22, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider21, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider20, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider19, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider18, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider17, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider16, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 3");
		ui_interface->addVerticalSlider("Input 0", &fVslider31, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider30, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider29, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider28, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider27, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider26, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider25, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider24, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 4");
		ui_interface->addVerticalSlider("Input 0", &fVslider39, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider38, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider37, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider36, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider35, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider34, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider33, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider32, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 5");
		ui_interface->addVerticalSlider("Input 0", &fVslider47, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider46, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider45, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider44, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider43, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider42, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider41, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider40, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 6");
		ui_interface->addVerticalSlider("Input 0", &fVslider55, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider54, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider53, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider52, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider51, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider50, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider49, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider48, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("Output 7");
		ui_interface->addVerticalSlider("Input 0", &fVslider63, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 1", &fVslider62, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 2", &fVslider61, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 3", &fVslider60, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 4", &fVslider59, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 5", &fVslider58, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 6", &fVslider57, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->addVerticalSlider("Input 7", &fVslider56, FAUSTFLOAT(-1e+01f), FAUSTFLOAT(-96.0f), FAUSTFLOAT(4.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->closeBox();
	}
	
	virtual void compute(int count, FAUSTFLOAT** RESTRICT inputs, FAUSTFLOAT** RESTRICT outputs) {
		FAUSTFLOAT* input0 = inputs[0];
		FAUSTFLOAT* input1 = inputs[1];
		FAUSTFLOAT* input2 = inputs[2];
		FAUSTFLOAT* input3 = inputs[3];
		FAUSTFLOAT* input4 = inputs[4];
		FAUSTFLOAT* input5 = inputs[5];
		FAUSTFLOAT* input6 = inputs[6];
		FAUSTFLOAT* input7 = inputs[7];
		FAUSTFLOAT* output0 = outputs[0];
		FAUSTFLOAT* output1 = outputs[1];
		FAUSTFLOAT* output2 = outputs[2];
		FAUSTFLOAT* output3 = outputs[3];
		FAUSTFLOAT* output4 = outputs[4];
		FAUSTFLOAT* output5 = outputs[5];
		FAUSTFLOAT* output6 = outputs[6];
		FAUSTFLOAT* output7 = outputs[7];
		float fSlow0 = std::pow(1e+01f, 0.05f * float(fVslider0));
		float fSlow1 = std::pow(1e+01f, 0.05f * float(fVslider1));
		float fSlow2 = std::pow(1e+01f, 0.05f * float(fVslider2));
		float fSlow3 = std::pow(1e+01f, 0.05f * float(fVslider3));
		float fSlow4 = std::pow(1e+01f, 0.05f * float(fVslider4));
		float fSlow5 = std::pow(1e+01f, 0.05f * float(fVslider5));
		float fSlow6 = std::pow(1e+01f, 0.05f * float(fVslider6));
		float fSlow7 = std::pow(1e+01f, 0.05f * float(fVslider7));
		float fSlow8 = std::pow(1e+01f, 0.05f * float(fVslider8));
		float fSlow9 = std::pow(1e+01f, 0.05f * float(fVslider9));
		float fSlow10 = std::pow(1e+01f, 0.05f * float(fVslider10));
		float fSlow11 = std::pow(1e+01f, 0.05f * float(fVslider11));
		float fSlow12 = std::pow(1e+01f, 0.05f * float(fVslider12));
		float fSlow13 = std::pow(1e+01f, 0.05f * float(fVslider13));
		float fSlow14 = std::pow(1e+01f, 0.05f * float(fVslider14));
		float fSlow15 = std::pow(1e+01f, 0.05f * float(fVslider15));
		float fSlow16 = std::pow(1e+01f, 0.05f * float(fVslider16));
		float fSlow17 = std::pow(1e+01f, 0.05f * float(fVslider17));
		float fSlow18 = std::pow(1e+01f, 0.05f * float(fVslider18));
		float fSlow19 = std::pow(1e+01f, 0.05f * float(fVslider19));
		float fSlow20 = std::pow(1e+01f, 0.05f * float(fVslider20));
		float fSlow21 = std::pow(1e+01f, 0.05f * float(fVslider21));
		float fSlow22 = std::pow(1e+01f, 0.05f * float(fVslider22));
		float fSlow23 = std::pow(1e+01f, 0.05f * float(fVslider23));
		float fSlow24 = std::pow(1e+01f, 0.05f * float(fVslider24));
		float fSlow25 = std::pow(1e+01f, 0.05f * float(fVslider25));
		float fSlow26 = std::pow(1e+01f, 0.05f * float(fVslider26));
		float fSlow27 = std::pow(1e+01f, 0.05f * float(fVslider27));
		float fSlow28 = std::pow(1e+01f, 0.05f * float(fVslider28));
		float fSlow29 = std::pow(1e+01f, 0.05f * float(fVslider29));
		float fSlow30 = std::pow(1e+01f, 0.05f * float(fVslider30));
		float fSlow31 = std::pow(1e+01f, 0.05f * float(fVslider31));
		float fSlow32 = std::pow(1e+01f, 0.05f * float(fVslider32));
		float fSlow33 = std::pow(1e+01f, 0.05f * float(fVslider33));
		float fSlow34 = std::pow(1e+01f, 0.05f * float(fVslider34));
		float fSlow35 = std::pow(1e+01f, 0.05f * float(fVslider35));
		float fSlow36 = std::pow(1e+01f, 0.05f * float(fVslider36));
		float fSlow37 = std::pow(1e+01f, 0.05f * float(fVslider37));
		float fSlow38 = std::pow(1e+01f, 0.05f * float(fVslider38));
		float fSlow39 = std::pow(1e+01f, 0.05f * float(fVslider39));
		float fSlow40 = std::pow(1e+01f, 0.05f * float(fVslider40));
		float fSlow41 = std::pow(1e+01f, 0.05f * float(fVslider41));
		float fSlow42 = std::pow(1e+01f, 0.05f * float(fVslider42));
		float fSlow43 = std::pow(1e+01f, 0.05f * float(fVslider43));
		float fSlow44 = std::pow(1e+01f, 0.05f * float(fVslider44));
		float fSlow45 = std::pow(1e+01f, 0.05f * float(fVslider45));
		float fSlow46 = std::pow(1e+01f, 0.05f * float(fVslider46));
		float fSlow47 = std::pow(1e+01f, 0.05f * float(fVslider47));
		float fSlow48 = std::pow(1e+01f, 0.05f * float(fVslider48));
		float fSlow49 = std::pow(1e+01f, 0.05f * float(fVslider49));
		float fSlow50 = std::pow(1e+01f, 0.05f * float(fVslider50));
		float fSlow51 = std::pow(1e+01f, 0.05f * float(fVslider51));
		float fSlow52 = std::pow(1e+01f, 0.05f * float(fVslider52));
		float fSlow53 = std::pow(1e+01f, 0.05f * float(fVslider53));
		float fSlow54 = std::pow(1e+01f, 0.05f * float(fVslider54));
		float fSlow55 = std::pow(1e+01f, 0.05f * float(fVslider55));
		float fSlow56 = std::pow(1e+01f, 0.05f * float(fVslider56));
		float fSlow57 = std::pow(1e+01f, 0.05f * float(fVslider57));
		float fSlow58 = std::pow(1e+01f, 0.05f * float(fVslider58));
		float fSlow59 = std::pow(1e+01f, 0.05f * float(fVslider59));
		float fSlow60 = std::pow(1e+01f, 0.05f * float(fVslider60));
		float fSlow61 = std::pow(1e+01f, 0.05f * float(fVslider61));
		float fSlow62 = std::pow(1e+01f, 0.05f * float(fVslider62));
		float fSlow63 = std::pow(1e+01f, 0.05f * float(fVslider63));
		for (int i0 = 0; i0 < count; i0 = i0 + 1) {
			float fTemp0 = float(input7[i0]);
			float fTemp1 = float(input6[i0]);
			float fTemp2 = float(input5[i0]);
			float fTemp3 = float(input4[i0]);
			float fTemp4 = float(input3[i0]);
			float fTemp5 = float(input2[i0]);
			float fTemp6 = float(input1[i0]);
			float fTemp7 = float(input0[i0]);
			output0[i0] = FAUSTFLOAT(fSlow7 * fTemp7 + fSlow6 * fTemp6 + fSlow5 * fTemp5 + fSlow4 * fTemp4 + fSlow3 * fTemp3 + fSlow2 * fTemp2 + fSlow1 * fTemp1 + fSlow0 * fTemp0);
			output1[i0] = FAUSTFLOAT(fSlow15 * fTemp7 + fSlow14 * fTemp6 + fSlow13 * fTemp5 + fSlow12 * fTemp4 + fSlow11 * fTemp3 + fSlow10 * fTemp2 + fSlow9 * fTemp1 + fSlow8 * fTemp0);
			output2[i0] = FAUSTFLOAT(fSlow23 * fTemp7 + fSlow22 * fTemp6 + fSlow21 * fTemp5 + fSlow20 * fTemp4 + fSlow19 * fTemp3 + fSlow18 * fTemp2 + fSlow17 * fTemp1 + fSlow16 * fTemp0);
			output3[i0] = FAUSTFLOAT(fSlow31 * fTemp7 + fSlow30 * fTemp6 + fSlow29 * fTemp5 + fSlow28 * fTemp4 + fSlow27 * fTemp3 + fSlow26 * fTemp2 + fSlow25 * fTemp1 + fSlow24 * fTemp0);
			output4[i0] = FAUSTFLOAT(fSlow39 * fTemp7 + fSlow38 * fTemp6 + fSlow37 * fTemp5 + fSlow36 * fTemp4 + fSlow35 * fTemp3 + fSlow34 * fTemp2 + fSlow33 * fTemp1 + fSlow32 * fTemp0);
			output5[i0] = FAUSTFLOAT(fSlow47 * fTemp7 + fSlow46 * fTemp6 + fSlow45 * fTemp5 + fSlow44 * fTemp4 + fSlow43 * fTemp3 + fSlow42 * fTemp2 + fSlow41 * fTemp1 + fSlow40 * fTemp0);
			output6[i0] = FAUSTFLOAT(fSlow55 * fTemp7 + fSlow54 * fTemp6 + fSlow53 * fTemp5 + fSlow52 * fTemp4 + fSlow51 * fTemp3 + fSlow50 * fTemp2 + fSlow49 * fTemp1 + fSlow48 * fTemp0);
			output7[i0] = FAUSTFLOAT(fSlow63 * fTemp7 + fSlow62 * fTemp6 + fSlow61 * fTemp5 + fSlow60 * fTemp4 + fSlow59 * fTemp3 + fSlow58 * fTemp2 + fSlow57 * fTemp1 + fSlow56 * fTemp0);
		}
	}

};

#include <stdio.h>
#include <string>
#include "m_pd.h"

#define sym(name) xsym(name)
#define xsym(name) #name
#define faust_setup(name) xfaust_setup(name)
#define xfaust_setup(name) name ## _tilde_setup(void)

// time for "active" toggle xfades in secs
#define XFADE_TIME 0.1f

static t_class *faust_class;

struct t_faust {
  t_object x_obj;
#ifdef __MINGW32__
  /* This seems to be necessary as some as yet undetermined Pd routine seems
     to write past the end of x_obj on Windows. */
  int fence; /* dummy field (not used) */
#endif
  mydsp *dsp;
  PdUI *ui;
  std::string *label;
  int active, xfade, n_xfade, rate, n_in, n_out;
  t_sample **inputs, **outputs, **buf;
  t_outlet *out;
  t_sample f;
};

static t_symbol *s_button, *s_checkbox, *s_vslider, *s_hslider, *s_nentry,
  *s_vbargraph, *s_hbargraph;

static inline void zero_samples(int k, int n, t_sample **out)
{
  for (int i = 0; i < k; i++)
#ifdef __STDC_IEC_559__
    /* IEC 559 a.k.a. IEEE 754 floats can be initialized faster like this */
    memset(out[i], 0, n*sizeof(t_sample));
#else
    for (int j = 0; j < n; j++)
      out[i][j] = 0.0f;
#endif
}

static inline void copy_samples(int k, int n, t_sample **out, t_sample **in)
{
  for (int i = 0; i < k; i++)
    memcpy(out[i], in[i], n*sizeof(t_sample));
}

static t_int *faust_perform(t_int *w)
{
  t_faust *x = (t_faust *)(w[1]);
  int n = (int)(w[2]);
  if (!x->dsp || !x->buf) return (w+3);
  AVOIDDENORMALS;
  if (x->xfade > 0) {
    float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
    d = d/n;
    x->dsp->compute(n, x->inputs, x->buf);
    if (x->active)
      if (x->n_in == x->n_out)
	/* xfade inputs -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->inputs[i][j]+(1.0f-f)*x->buf[i][j];
      else
	/* xfade 0 -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = (1.0f-f)*x->buf[i][j];
    else
      if (x->n_in == x->n_out)
	/* xfade buf -> inputs */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j]+(1.0f-f)*x->inputs[i][j];
      else
	/* xfade buf -> 0 */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j];
  } else if (x->active) {
    x->dsp->compute(n, x->inputs, x->buf);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else if (x->n_in == x->n_out) {
    copy_samples(x->n_out, n, x->buf, x->inputs);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else
    zero_samples(x->n_out, n, x->outputs);
  return (w+3);
}

static void faust_dsp(t_faust *x, t_signal **sp)
{
  int n = sp[0]->s_n, sr = (int)sp[0]->s_sr;
  if (x->rate <= 0) {
    /* default sample rate is whatever Pd tells us */
    PdUI *ui = x->ui;
    float *z = NULL;
    if (ui->nelems > 0 &&
	(z = (float*)malloc(ui->nelems*sizeof(float)))) {
      /* save the current control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  z[i] = *ui->elems[i].zone;
    }
    /* set the proper sample rate; this requires reinitializing the dsp */
    x->rate = sr;
    x->dsp->init(sr);
    if (z) {
      /* restore previous control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  *ui->elems[i].zone = z[i];
      free(z);
    }
  }
  if (n > 0)
    x->n_xfade = (int)(x->rate*XFADE_TIME/n);
  dsp_add(faust_perform, 2, x, n);
  for (int i = 0; i < x->n_in; i++)
    x->inputs[i] = sp[i+1]->s_vec;
  for (int i = 0; i < x->n_out; i++)
    x->outputs[i] = sp[x->n_in+i+1]->s_vec;
  if (x->buf != NULL)
    for (int i = 0; i < x->n_out; i++) {
      x->buf[i] = (t_sample*)malloc(n*sizeof(t_sample));
      if (x->buf[i] == NULL) {
	for (int j = 0; j < i; j++)
	  free(x->buf[j]);
	free(x->buf);
	x->buf = NULL;
	break;
      }
    }
}

static int pathcmp(const char *s, const char *t)
{
  int n = strlen(s), m = strlen(t);
  if (n == 0 || m == 0)
    return 0;
  else if (t[0] == '/')
    return strcmp(s, t);
  else if (n <= m || s[n-m-1] != '/')
    return strcmp(s+1, t);
  else
    return strcmp(s+n-m, t);
}

static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!x->dsp) return;
  PdUI *ui = x->ui;
  if (s == &s_bang) {
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label && ui->elems[i].zone) {
	t_atom args[6];
	t_symbol *_s;
	switch (ui->elems[i].type) {
	case UI_BUTTON:
	  _s = s_button;
	  break;
	case UI_CHECK_BUTTON:
	  _s = s_checkbox;
	  break;
	case UI_V_SLIDER:
	  _s = s_vslider;
	  break;
	case UI_H_SLIDER:
	  _s = s_hslider;
	  break;
	case UI_NUM_ENTRY:
	  _s = s_nentry;
	  break;
	case UI_V_BARGRAPH:
	  _s = s_vbargraph;
	  break;
	case UI_H_BARGRAPH:
	  _s = s_hbargraph;
	  break;
	default:
	  continue;
	}
	SETSYMBOL(&args[0], gensym(ui->elems[i].label));
	SETFLOAT(&args[1], *ui->elems[i].zone);
	SETFLOAT(&args[2], ui->elems[i].init);
	SETFLOAT(&args[3], ui->elems[i].min);
	SETFLOAT(&args[4], ui->elems[i].max);
	SETFLOAT(&args[5], ui->elems[i].step);
	outlet_anything(x->out, _s, 6, args);
      }
  } else {
    const char *label = s->s_name;
    int count = 0;
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label &&
	  pathcmp(ui->elems[i].label, label) == 0) {
	if (argc == 0) {
	  if (ui->elems[i].zone) {
	    t_atom arg;
	    SETFLOAT(&arg, *ui->elems[i].zone);
	    outlet_anything(x->out, gensym(ui->elems[i].label), 1, &arg);
	  }
	  ++count;
	} else if (argc == 1 &&
		   (argv[0].a_type == A_FLOAT ||
		    argv[0].a_type == A_DEFFLOAT) &&
		   ui->elems[i].zone) {
	  float f = atom_getfloat(argv);
	  *ui->elems[i].zone = f;
	  ++count;
	} else
	  pd_error(x, "[faust] %s: bad control argument: %s",
		   x->label->c_str(), label);
      }
    if (count == 0 && strcmp(label, "active") == 0) {
      if (argc == 0) {
	t_atom arg;
	SETFLOAT(&arg, (float)x->active);
	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
      } else if (argc == 1 &&
		 (argv[0].a_type == A_FLOAT ||
		  argv[0].a_type == A_DEFFLOAT)) {
	float f = atom_getfloat(argv);
	x->active = (int)f;
	x->xfade = x->n_xfade;
      }
    }
  }
}

static void faust_free(t_faust *x)
{
  if (x->label) delete x->label;
  if (x->dsp) delete x->dsp;
  if (x->ui) delete x->ui;
  if (x->inputs) free(x->inputs);
  if (x->outputs) free(x->outputs);
  if (x->buf) {
    for (int i = 0; i < x->n_out; i++)
      if (x->buf[i]) free(x->buf[i]);
    free(x->buf);
  }
}

static void *faust_new(t_symbol *s, int argc, t_atom *argv)
{
  t_faust *x = (t_faust*)pd_new(faust_class);
  int sr = -1;
  t_symbol *id = NULL;
  x->active = 1;
  for (int i = 0; i < argc; i++)
    if (argv[i].a_type == A_FLOAT || argv[i].a_type == A_DEFFLOAT)
      sr = (int)argv[i].a_w.w_float;
    else if (argv[i].a_type == A_SYMBOL || argv[i].a_type == A_DEFSYMBOL)
      id = argv[i].a_w.w_symbol;
  x->rate = sr;
  if (sr <= 0) sr = 44100;
  x->xfade = 0; x->n_xfade = (int)(sr*XFADE_TIME/64);
  x->inputs = x->outputs = x->buf = NULL;
    x->label = new std::string(sym(mydsp) "~");
  x->dsp = new mydsp();
  x->ui = new PdUI(sym(mydsp), id?id->s_name:NULL);
  if (!x->dsp || !x->ui || !x->label) goto error;
  if (id) {
    *x->label += " ";
    *x->label += id->s_name;
  }
  x->n_in = x->dsp->getNumInputs();
  x->n_out = x->dsp->getNumOutputs();
  if (x->n_in > 0)
    x->inputs = (t_sample**)malloc(x->n_in*sizeof(t_sample*));
  if (x->n_out > 0) {
    x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
    x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
  }
  if ((x->n_in > 0 && x->inputs == NULL) ||
      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
    goto error;
  for (int i = 0; i < x->n_out; i++)
    x->buf[i] = NULL;
  x->dsp->init(sr);
  x->dsp->buildUserInterface(x->ui);
  for (int i = 0; i < x->n_in; i++)
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
  x->out = outlet_new(&x->x_obj, 0);
  for (int i = 0; i < x->n_out; i++)
    outlet_new(&x->x_obj, &s_signal);
  return (void *)x;
 error:
  faust_free(x);
  x->dsp = NULL; x->ui = NULL;
  x->inputs = x->outputs = x->buf = NULL;
  return (void *)x;
}

extern "C" void faust_setup(mydsp)
{
  t_symbol *s = gensym(sym(mydsp) "~");
  faust_class =
    class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
	      sizeof(t_faust), CLASS_DEFAULT,
	      A_GIMME, A_NULL);
  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
  class_addanything(faust_class, faust_any);
  class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
  s_button = gensym((char*)"button");
  s_checkbox = gensym((char*)"checkbox");
  s_vslider = gensym((char*)"vslider");
  s_hslider = gensym((char*)"hslider");
  s_nentry = gensym((char*)"nentry");
  s_vbargraph = gensym((char*)"vbargraph");
  s_hbargraph = gensym((char*)"hbargraph");
  /* give some indication that we're loaded and ready to go */
  mydsp dsp = mydsp();
  post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
       dsp.getNumInputs(), dsp.getNumOutputs());
}

#endif
